﻿namespace SampleProject
{
    using Infragistics.Win.UltraWinGrid;
    using System.Drawing;
    using System.Windows.Forms;
    using System;
    using System.Linq;

    public partial class Form1 : Form
    {
        private bool checkboxChecked = false;

        public Form1()
        {
            InitializeComponent();           
            this.ultraGrid1.AfterSelectChange += UltraGrid1_AfterSelectChange;            
            this.ultraGrid1.BeforeSummaryDialog += UltraGrid1_BeforeSummaryDialog;
        }

        private void ultraGrid1_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {           
            
        }

        private void UltraGrid1_BeforeSummaryDialog(object sender, BeforeSummaryDialogEventArgs e)
        {
            if (e.Column.Key == "Price")
            {                
                var summaryDialog = e.SummaryDialog;
                var containerControl = summaryDialog.Controls[0].Controls[0].Controls[1] as Panel;

                var image = summaryDialog.ImageSum;
               
                //Remove the Default Sum Summary
                containerControl.Controls.RemoveAt(0);

                var checkBox = new CheckBox();
                checkBox.CheckAlign = System.Drawing.ContentAlignment.MiddleLeft;

                checkBox.Image = image;
                checkBox.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
                
                checkBox.Text = "Custom Sum";
                checkBox.TextAlign = System.Drawing.ContentAlignment.MiddleRight;

                var y = containerControl.Controls.Cast<Control>().Max(c => c.Bounds.Bottom);
                var x = containerControl.Controls[containerControl.Controls.Count - 1].Bounds.X;
                checkBox.Location = new Point(x, y);
                checkBox.Visible = true;

                checkBox.Checked = checkboxChecked;
                checkBox.CheckedChanged += checkBox_CheckedChanged;                

                containerControl.Controls.Add(checkBox);

                var height = checkBox.Height;
                containerControl.Height += height;
                summaryDialog.Height += height;

                var buttonCancel = summaryDialog.Controls[0].Controls[0].Controls[2];
                var buttonOk = summaryDialog.Controls[0].Controls[0].Controls[3];

                buttonOk.Location = new Point(buttonOk.Location.X, buttonOk.Location.Y + height);
                buttonCancel.Location = new Point(buttonCancel.Location.X, buttonCancel.Location.Y + height);

                summaryDialog.FormClosing += summaryDialog_FormClosing;                
            }
        }        

        private void summaryDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            var form = sender as Form;
            var containerControl = form.Controls[0].Controls[0].Controls[1] as Panel;
            var customSumControl = containerControl.Controls.Cast<Control>().Where(c => c.Text == "Custom Sum").FirstOrDefault();

            if (customSumControl != null)
            {
                var customSumCheckBox = customSumControl as CheckBox;

                //Check if Custom Summary Exists Already
                var summary = this.ultraGrid1.DisplayLayout.Bands[0].Summaries.Cast<SummarySettings>().Where(s => s.Key == "CustomSum").FirstOrDefault();
                var gridBand = this.ultraGrid1.DisplayLayout.Bands[0];

                //Add Custom Summary if the checkbox is checked and it is not created
                if (customSumCheckBox.Checked && form.DialogResult == DialogResult.OK && summary == null)
                {                    
                    SummarySettings ss = gridBand.Summaries.Add(SummaryType.Sum, gridBand.Columns["Price"]);
                    ss.SummaryType = SummaryType.Custom;
                    ss.Key = "CustomSum";
                    ss.CustomSummaryCalculator = new SummaryCalculator();
                }

                //Remove Custom Summary if the checkbox is unchecked and it is already created
                if (!customSumCheckBox.Checked && form.DialogResult == DialogResult.OK && summary != null)
                {
                    gridBand.Summaries.Remove(summary);
                }
                
                form.FormClosing -= summaryDialog_FormClosing;
            }           
        }

        private void checkBox_CheckedChanged(object sender, EventArgs e)
        {           
            checkboxChecked = ((CheckBox)sender).Checked;
        }
        

        private void UltraGrid1_AfterSelectChange(object sender, AfterSelectChangeEventArgs e)
        {
            UltraGrid grid = (UltraGrid)sender;
            if (grid.DisplayLayout.Bands[0].Summaries.Count > 0)
            {
                grid.DisplayLayout.Bands[0].Summaries[0].Refresh();
            } 
        }
    }

    public class SummaryCalculator : ICustomSummaryCalculator
    {
        private double total = 0;

        // Begins the summary for the SummarySettings object passed in. 
        public void BeginCustomSummary(SummarySettings settings, RowsCollection rows)
        {
            this.total = 0;
        }

        // Process each row that gets passed in
        public void AggregateCustomSummary(SummarySettings settings, UltraGridRow row)
        {
            if (row.Selected)
            {
                string cellValue = row.GetCellText(settings.SourceColumn);
                    
                double val = 0;
                if (double.TryParse(cellValue, out val))
                    this.total += val;
            }
        }

        // Return the calculated summary value
        public object EndCustomSummary(SummarySettings settings, RowsCollection rows)
        {
            return this.total;
        }
    }
}
